function [koeff_mat,inhom_vec,X,Y,G,index_in]=koeffmat_inhom(punkte_anz,varargin)
% Berechnet die Koeffizienten Matrix und den zugehrigen
% Inhomogenittsvektor des linearen Gleichungssystems
% -----------------------------------------------------------------------
  % Eingabeparameter:
  % ----------------
  % - punkte_anz ... berechnete Punkteanzahl (vom 'slider')
  %
  % - varargin{1} ... 2-dim Array 1. Sp.: Anzahl der Unterteilungen
  %                                         entlang der x-Achse
  %                               2. Sp.: zugehoerige Punkteanzahl
  %
% -----------------------------------------------------------------------
  % Ausgabeparameter:
  % ----------------
  % - koeff_mat ... Koeffizientenmatrix
  % - inhom_vec ... Inhomogenitaetsvektor
  % - X...Matrix, die die x-Koordinaten fuer jede y-Koordinate enthaelt
  % - Y...Matrix, die die y-Koordinaten fuer jede x-Koordinate enthaelt
  % - G...Betrachtetes Gebiet der inneren Punkte  
  % - index_in ...Indices der 'inneren' Punkte in G
% -----------------------------------------------------------------------  
 
if nargin < 2,
  error('koeffmat_inhom: Not enough input parameters!');  
elseif nargin < 3,
  points = temp_int_point; 
  berech_stat = varargin{1};
elseif nargin == 3,
  points = varargin{1};
  berech_stat = varargin{2};
elseif nargin < 6,
  points = temp_int_point;           % Anzahl der Punkte im inneren des Gebietes!
  berech_stat = varargin{1};         % Berechungsstatus: vorgegebene Randbed. bzw. bel. Randbedingungen
  interp_stat = varargin{2};         % Art der Interpolation zwischen den vorgegebenen Temperaturen 
				     % auf dem Rand an bestimmten Punkten
  l_bound = varargin{3};             % Laenge entlang des Randes (Ausgangspunkt: x=R, y=0)
  temp_bound = varargin{4};          % Temperatur auf dem Rand an vorgegebenen Punkten!
elseif nargin == 6
  points = varargin{1};
  berech_stat = varargin{2};
  interp_stat = varargin{3};
  l_bound = varargin{4};
  temp_bound = varargin{5};
end

R=4;                % Radius des Halbkreises, sowie die Seitenlaenge des unteren Quadrats!
index_p = min(find(points(:,2) == punkte_anz));
int_anz = points(index_p,1);       % Anzahl der Intervalle entlang der
                                   % x-Achse (Achse des Halbkreises)

h = 2*R/int_anz;                   % Gitterkonstante
x = -R:h:R;
y = R:-h:-R;
[X,Y] = meshgrid(x,y);             

% Koordinaten der inneren Punkte
G1 = ( (R > sqrt(X.^2+Y.^2)) & (Y > 0) );
G2 = ( (X > 0) & (X < R) & (Y <= 0) & (Y > -R) );
G = G1 + G2;

index_in=find(G);
vec_lg = length(index_in); % entspricht Anzahl der inneren Punkte
XG = X.*G;
YG = Y.*G;

XG_vec=X(index_in);              
YG_vec=Y(index_in);

% x-Koordinaten der inneren Punkte im Halbkreis
XG1_vec = XG_vec.*(YG_vec>0);
YG1_vec = YG_vec.*(YG_vec>0);

% x-Koordinaten der inneren Punkte im Quadrat
XG2_vec = XG_vec.*(YG_vec<=0);
YG2_vec = YG_vec.*(YG_vec<=0);


% --------------------Punkte ohne linken Nachbarn--------------------
RGol = sqrt((XG1_vec-h).^2+YG1_vec.^2);
ind1_vec_ol = (RGol >= R);                 % die Positionsnummer wird auf 1 gesetzt, wenn Bedingung zutrifft, sonst 0!
if ~mod(int_anz,2)     % fuer geradzahliges "int_anz"
  ind2_vec_ol = ((XG2_vec - h) >= -eps & (XG2_vec - h) <= eps);
else                   % fuer ungeradzahliges "int_anz"
  ind2_vec_ol = ((XG2_vec - h/2) >= -eps & (XG2_vec - h/2) <= eps);
end
ind_vec_ol = ind1_vec_ol + ind2_vec_ol;

x_ol = -sqrt(R^2 - YG_vec.^2).*ind1_vec_ol;     % x-Koordinaten des linken Teiles des Halbkreises

delta1_ol = abs(mod(x_ol,XG1_vec));             % Abstand zum linken Rand des Halbkreises
delta2_ol = h*ind2_vec_ol;                      % Abstand zum linken Rand des Quadrats wird immer
                                                % h gesetzt, damit vereinfacht sich die Berechnung 
                                                % fuer den Inhomogenitaetsvektor (siehe Berechnung!)
delta_ol = delta1_ol + delta2_ol; 

if berech_stat == 1       % Testbeispiel: bekannte analytische Loesung
  % Randwert zu den Punkten ohne linken Nachbarn
  phi_ol = phi_AB(x_ol.*ind1_vec_ol,YG1_vec) + phi_BD(YG2_vec).*ind2_vec_ol;          
elseif berech_stat == 2   % bel. Randbedingung, festgelegt an bestimmten
                          % Punkten, und entsprechend interpoliert
  win1_ol = atan2(YG1_vec,x_ol).*ind1_vec_ol;     % Winkel im Halbkreis (2. Quadrant)
  l1_ol = R*win1_ol;                              
  l2_ol = (R*(pi+1)-YG_vec.*ind2_vec_ol).*ind2_vec_ol;
  l_ol = l1_ol + l2_ol;     % Bogenlaenge, fuer Punkte ohne rechten Nachbarn
  
  if interp_stat == 1       % lineare Interpolation
    phi_ol = interp1(l_bound,temp_bound,l_ol,'linear').*ind_vec_ol;
  elseif interp_stat == 2   % 'Piecewise cubic Hermite interpolation'
    phi_ol = interp1(l_bound,temp_bound,l_ol,'pchip').*ind_vec_ol;
  end
  
end 


% --------------------Punkte ohne rechten Nachbarn--------------------
RGor = sqrt((XG1_vec + h).^2 + YG1_vec.^2);
ind1_vec_or = (RGor >= R);
ind2_vec_or = (((XG2_vec + h) >= R-eps) & ((XG2_vec + h) <= R+eps));
ind_vec_or = ind1_vec_or + ind2_vec_or;

x_or = sqrt(R^2 - YG_vec.^2).*ind1_vec_or;     % x-Koordinaten des linken Teiles des Halbkreises
delta1_or = mod(x_or,XG1_vec);
delta2_or = h*ind2_vec_or;
delta_or = delta1_or + delta2_or;

if berech_stat == 1
  % Randwert zu den Punkten ohne rechten Nachbarn
  phi_or = phi_AB(x_or,YG1_vec) + phi_DA(YG2_vec).*ind2_vec_or;  
elseif berech_stat == 2   % bel. Randbedingung, festgelegt an bestimmten
                          % Punkten, und entsprechend interpoliert
  win1_or = atan2(YG1_vec,x_or).*ind1_vec_or;     % Winkel im Halbkreis (1. Quadrant)
  l1_or = R*win1_or;                              
  l2_or = (R*(pi+4)+YG_vec.*ind2_vec_or).*ind2_vec_or;
  l_or = l1_or + l2_or;                           % Bogenlaenge, fuer
                                                  % Punkte ohne rechten Nachbarn
  if interp_stat == 1   % lineare Interpolation
    phi_or = interp1(l_bound,temp_bound,l_or,'linear').*ind_vec_or;
  elseif interp_stat == 2   % 'Piecewise cubic Hermite interpolation'
    phi_or = interp1(l_bound,temp_bound,l_or,'pchip').*ind_vec_or;
  end
end


% --------------------Punkte ohne oberen Nachbarn--------------------
RGoo = sqrt(XG1_vec.^2 + (YG1_vec + h).^2);
ind_vec_oo = (RGoo >= R);

y_oo = sqrt(R^2 - XG1_vec.^2).*ind_vec_oo;   % y-Werte zum oberen Halbkreisrand an den entsprechenden Stellen
delta_oo = mod(y_oo,YG1_vec);

if berech_stat == 1
  % Randwert zu den Punkten ohne oberen Nachbarn
  phi_oo = phi_AB(XG1_vec,y_oo);      
elseif berech_stat == 2   % bel. Randbedingung, festgelegt an bestimmten
                          % Punkten, und entsprechend interpoliert
  win_oo = atan2(y_oo,XG1_vec).*ind_vec_oo;     % Winkel im Halbkreis (1. und 2. Quadrant)
  l_oo = R*win_oo;                              % Bogenlaenge, gemessen von x=R und y=0!
  if interp_stat == 1       % lineare Interpolation
    phi_oo = interp1(l_bound,temp_bound,l_oo,'linear').*ind_vec_oo;
  elseif interp_stat == 2   % 'Piecewise cubic Hermite interpolation'
    phi_oo = interp1(l_bound,temp_bound,l_oo,'pchip').*ind_vec_oo;
  end
end

% --------------------Punkte ohne unteren Nachbarn--------------------
if ~mod(int_anz,2)    % oberes linkes Gebiet fuer geradzahliges "int_anz"                        
  ind1_vec_ou = (((YG1_vec - h) >= -eps) & ((YG1_vec - h) <= eps) & (XG_vec <= 0));  
else                  % oberes linkes Gebiet fuer ungeradzahliges "int_anz"
  ind1_vec_ou = (((YG1_vec - h/2) >= -eps) & ((YG1_vec - h/2) <= eps) & (XG_vec <= 0));
end
ind2_vec_ou = (((YG2_vec-h) <= -(R-eps)) & ((YG2_vec-h) >= -(R+eps)));              % unteres Gebiet (= Quadrat)
ind_vec_ou = ind1_vec_ou + ind2_vec_ou;

delta1_ou = h*ind1_vec_ou;
delta2_ou = h*ind2_vec_ou;
delta_ou = delta1_ou + delta2_ou;

if berech_stat == 1
  % Randwert zu den Punkten ohne unteren Nachbarn  
  phi_ou = phi_BD(XG_vec).*ind1_vec_ou + phi_DA(XG_vec).*ind2_vec_ou;  
  
elseif berech_stat == 2   % bel. Randbedingung, festgelegt an bestimmten
                          % Punkten, und entsprechend interpoliert
  l1_ou = (R*(pi+1) + XG_vec).*ind1_vec_ou;   
  l2_ou = (R*(pi+2) + XG_vec).*ind2_vec_ou;
  l_ou = l1_ou + l2_ou;     % Bogenlaenge fuer Punkte ohne untere Nachbarn
  if interp_stat == 1       % lineare Interpolation
    phi_ou = interp1(l_bound,temp_bound,l_ou,'linear').*ind_vec_ou;
  elseif interp_stat == 2   % 'Piecewise cubic Hermite interpolation'
    phi_ou = interp1(l_bound,temp_bound,l_ou,'pchip').*ind_vec_ou;
  end
end


%---------------------------------------------------------------------

% Initialisiere Inhomogenitaetsvektor und die Diagonale der Koeffizientenmatrix
inhom_vec = zeros(vec_lg,1);
diag_vec= 4*ones(vec_lg,1);

dummy = zeros(vec_lg,1);      % Hilfsvariable

% Veraenderung des Inhomogenitaetsvektors und der Diagonale aufgrund 
% der linken fehlenden Punkte
dummy(find(delta_ol)) = h./delta_ol(find(delta_ol));  % wird auch fuer Punkte im Quadrat berechnet;
                                                      % da der Abstand zum Rand immer h gesetzt wird,
                                                      % kuerzt er sich raus, wodurch nur der Randwert
                                                      % Phi selbst eingeht!
inhom_vec = dummy.*phi_ol;
diag_vec = diag_vec - (1 - dummy).*ind1_vec_ol;   % nur im Halbkreis muss (1-h/delta) von
                                                  % der Diagonale der "koeff_mat" abgezogen werden!

% aufgrund der rechts fehlenden Punkte
dummy=0*dummy;
dummy(find(delta_or)) = h./delta_or(find(delta_or));
inhom_vec = inhom_vec + dummy.*phi_or;
diag_vec = diag_vec - (1 - dummy).*ind1_vec_or;

% aufgrund der oben fehlenden Punkte
dummy=0*dummy;
dummy(find(delta_oo)) = h./delta_oo(find(delta_oo));
inhom_vec = inhom_vec + dummy.*phi_oo;
diag_vec = diag_vec - (1 - dummy).*ind_vec_oo;

% aufgrund der unten fehlenden Punkte
inhom_vec = inhom_vec + phi_ou;

%------------------------------------------------------------------------------
% Koeffizienten Matrix unter Beruecksichtigung der Punkte ohne
% Nachbarn (es fehlt noch die der Punkte mit Nachbarn)
koeff_mat = diag(diag_vec);
cat(2,(1:vec_lg)',diag_vec,inhom_vec);

%return
vgl=cat(2,ind_vec_ol,ind_vec_or,ind_vec_oo,ind_vec_ou);   % Matrix, deren Spalten eine 1 enthalten, wenn kein Nachbarvorhanden
[Z,S] = find(G);
nr_ind = cat(2,(1:vec_lg)',Z,S);                  % enthaelt die Punkt-Nummer, den Zeilen- und Spaltenindex


%-----------------Bestimmung der Nachbarn und entsprechende Aenderung der "koeff_mat"---
if int_anz >= 5
  for c = 1:vec_lg
    if vgl(c,:)== [0 0 0 0] % alle Nachbarn vorhanden
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb r_nb o_nb u_nb]) = -1;
    elseif vgl(c,:)==[1 0 0 0]  % keinen linken Nachbarn
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[r_nb o_nb u_nb]) = -1;
    elseif vgl(c,:)==[0 1 0 0]  % keinen rechten Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb o_nb u_nb]) = -1;
    elseif vgl(c,:)==[0 0 1 0]  % keinen oberen Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb r_nb u_nb]) = -1;
    elseif vgl(c,:)==[0 0 0 1]  % keinen unteren Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb r_nb o_nb]) = -1;
    elseif vgl(c,:)==[1 0 1 0]  % keinen linken und keinen oberen Nachbarn
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[r_nb u_nb]) = -1;
    elseif vgl(c,:)==[1 0 0 1]  % keinen linken und keinen unteren Nachbarn
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[r_nb o_nb]) = -1;
    elseif vgl(c,:)==[0 1 1 0]  % keinen rechten und keinen oberen Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb u_nb]) = -1;
    elseif vgl(c,:)==[0 1 0 1]  % keinen rechten und keinen unteren Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb o_nb]) = -1;
    end
  end
elseif (int_anz == 3) | (int_anz == 4)  % Ausnahmen bei Intervallanzahl 3 und 4!
  for c = 1:vec_lg
    if vgl(c,:)==[1 0 1 1]      % keinen linken, keinen oberen und keinen unteren Nachbarn
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      koeff_mat(c,r_nb) = -1;
    elseif vgl(c,:)==[1 1 0 1]  % keinen linken ,keinen unteren und keinen rechten Nachbarn
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,o_nb) = -1;
    elseif vgl(c,:)==[0 0 1 1]  % keinen oberen und keinen unteren Nachbar 
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      r_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)+1) );
      koeff_mat(c,[l_nb r_nb]) = -1;
    elseif vgl(c,:)==[1 1 0 0]  % keinen linken und keinen rechten Nachbar
      o_nb = find( (nr_ind(:,2)==nr_ind(c,2)-1) & (nr_ind(:,3)==nr_ind(c,3)) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[o_nb u_nb]) = -1;
    elseif vgl(c,:)==[0 1 1 0]  % keinen rechten und keinen oberen Nachbarn
      l_nb = find( (nr_ind(:,2)==nr_ind(c,2)) & (nr_ind(:,3)==nr_ind(c,3)-1) );
      u_nb = find( (nr_ind(:,2)==nr_ind(c,2)+1) & (nr_ind(:,3)==nr_ind(c,3)) );
      koeff_mat(c,[l_nb u_nb]) = -1;
    end
  end   
end  % Ende der for-Schleife fuer Bestimmung der 'koeff_mat'









